#!/bin/bash
set -e

self_="$(readlink -f "$0")"
main_dir="${self_%/*/*}"
split_="$main_dir/otsplit"
extract_="$main_dir/otextract"

mkdir -p "$main_dir"/tests/tmp
cd "$main_dir"/tests/tmp

set +e

rm -f foo.*
"$split_" -s foo.{1..7} < "$self_"
"$split_" -j foo.* > copy.txt
if ! diff -q "$self_" copy.txt
then
  echo "error: random split failed"
  exit 1
else
  echo "random split succeeded"
fi
rm foo.*

dd if=/dev/urandom of=pad bs=1M count=5 2>/dev/null

rm -f foo
"$split_" -s -p pad foo < "$self_"
"$split_" -j -p pad foo > copy.txt
if ! diff -q "$self_" copy.txt
then
  echo "error: pre-existing pad split failed"
  exit 1
else
  echo "pre-existing pad split succeeded"
fi
rm foo

"$split_" -s -p pad - < "$self_" | "$split_" -j - pad > copy.txt
if ! diff -q "$self_" copy.txt
then
  echo "error: piping failed"
  exit 1
else
  echo "piping succeeded"
fi

"$split_" -s -o 15678 -p pad - < "$self_" | "$split_" -j -o 15678 - pad > copy.txt
if ! diff -q "$self_" copy.txt
then
  echo "error: offset failed"
  exit 1
else
  echo "offset succeeded"
fi

"$split_" -s -r /dev/zero foo.{1..2} < "$self_"
"$split_" -j foo.* > copy.txt
if ! diff -q "$self_" copy.txt
then
  echo "error: input source option failed"
  exit 1
else
  echo "input source option succeeded"
fi
rm foo.*

rm -f page.*
"$extract_" -r "$self_" pad page.1
"$split_" -s -p page.1 foo < "$self_"
"$extract_" -xr foo pad page.2
"$split_" -j -p page.2 foo > copy.txt
if ! diff -q "$self_" copy.txt
then
  echo "error: page extraction failed"
  exit 1
else
  echo "page extraction succeeded"
fi
rm page.*







# TODO
# Add more test cases (tedious).
echo -n "abcde" > abc.1
"$extract_" -l 3 abc.1 abc.2
if [[ $(cat abc.1) != "abcde" ]] \
|| [[ $(cat abc.2) != "abc" ]]
then
  echo "error: extract by length failed"
  exit 1
else
  echo "extract by length succeeded"
fi

"$extract_" -xl 3 abc.1 abc.2
if [[ $(cat abc.1) != "de" ]] \
|| [[ $(cat abc.2) != "abc" ]]
then
  echo "error: extract by length with excision failed"
  exit 1
else
  echo "extract by length with excision succeeded"
fi


echo -n "abcde" > abc.1
"$extract_" -l -3 abc.1 abc.2
if [[ $(cat abc.1) != "abcde" ]] \
|| [[ $(cat abc.2) != "cde" ]]
then
  echo "error: extract by negative length failed"
  exit 1
else
  echo "extract by negative length succeeded"
fi

"$extract_" -xl -3 abc.1 abc.2
if [[ $(cat abc.1) != "ab" ]] \
|| [[ $(cat abc.2) != "cde" ]]
then
  echo "error: extract by negative length with excision failed"
  exit 1
else
  echo "extract by negative length with excision succeeded"
fi







# cd --
echo "all tests passed"
exit 0
